@@ -15,7 +15,7 @@
 #' @param rule The condition under which to apply the formatting. See examples.
 #' @param style A style to apply to those cells that satisfy the rule. Default is createStyle(fontColour = "#9C0006", bgFill = "#FFC7CE")
 #' @param type Either 'expression', 'colourScale', 'databar', 'duplicates', 'beginsWith', 
-#' 'endsWith', 'contains' or 'notContains' (case insensitive).
+#' 'endsWith', 'topN', 'bottomN', 'contains' or 'notContains' (case insensitive).
 #' @param ... See below
 #' @details See Examples.
 #'
@@ -62,6 +62,30 @@
 #'   \item{rule is a numeric vector of length 2 specifying lower and upper bound (Inclusive)}
 #' }
 #'
+#' If type == "topN"
+#' \itemize{
+#'   \item{style is a Style object. See \code{\link{createStyle}}}
+#'   \item{rule is ignored}
+#'   \item{...
+#'   \itemize{
+#'     \item{\bold{rank} numeric vector of length 1 indicating number of highest values.}
+#'     \item{\bold{percent} TRUE if you want top N percentage.}
+#'      }
+#'    }
+#' }
+#' 
+#' If type == "bottomN"
+#' \itemize{
+#'   \item{style is a Style object. See \code{\link{createStyle}}}
+#'   \item{rule is ignored}
+#'   \item{...
+#'   \itemize{
+#'     \item{\bold{rank} numeric vector of length 1 indicating number of lowest values.}
+#'     \item{\bold{percent} TRUE if you want bottom N percentage.}
+#'      }
+#'    }
+#' }
+#' 
 #' @seealso \code{\link{createStyle}}
 #' @export
 #' @examples
@@ -78,6 +102,8 @@
 #' addWorksheet(wb, "colourScale", zoom = 30)
 #' addWorksheet(wb, "databar")
 #' addWorksheet(wb, "between")
+#' addWorksheet(wb, "topN")
+#' addWorksheet(wb, "bottomN")
 #' addWorksheet(wb, "logical operators")
 #'
 #' negStyle <- createStyle(fontColour = "#9C0006", bgFill = "#FFC7CE")
@@ -190,13 +216,31 @@
 #' writeData(wb, "databar", -5:5)
 #' conditionalFormatting(wb, "databar", cols = 1, rows = 1:11, type = "databar") ## Default colours
 #'
-#' ## Betweem
+#' ## Between
 #' # Highlight cells in interval [-2, 2]
 #' writeData(wb, "between", -5:5)
 #' conditionalFormatting(wb, "between", cols = 1, rows = 1:11, type = "between", rule = c(-2, 2))
 #'
+#' ## Top N 
+#' writeData(wb, "topN", data.frame(x = 1:10, y = rnorm(10)))
+#' # Highlight top 5 values in column x
+#' conditionalFormatting(wb, "topN", cols = 1, rows = 2:11, 
+#'  style = posStyle, type = "topN", rank = 5)#'
+#' # Highlight top 20 percentage in column y
+#' conditionalFormatting(wb, "topN", cols = 2, rows = 2:11, 
+#'  style = posStyle, type = "topN", rank = 20, percent = TRUE)
+#'
+#'## Bottom N 
+#' writeData(wb, "bottomN", data.frame(x = 1:10, y = rnorm(10)))
+#' # Highlight bottom 5 values in column x
+#' conditionalFormatting(wb, "bottomN", cols = 1, rows = 2:11, 
+#'  style = negStyle, type = "topN", rank = 5)
+#' # Highlight bottom 20 percentage in column y
+#' conditionalFormatting(wb, "bottomN", cols = 2, rows = 2:11, 
+#'  style = negStyle, type = "topN", rank = 20, percent = TRUE)
+#'
 #' ## Logical Operators
-#' # You can use Excels logical Opertors
+#' # You can use Excels logical Operators
 #' writeData(wb, "logical operators", 1:10)
 #' conditionalFormatting(wb, "logical operators",
 #'   cols = 1, rows = 1:10,
@@ -273,6 +317,10 @@ conditionalFormatting <-
       type <- "endsWith"
     } else if (type == "between") {
       type <- "between"
+    } else if (type == "topn") {
+      type <- "topN"
+    } else if (type == "bottomn") {
+      type <- "bottomN"
     } else if (type != "expression") {
       stop(
         "Invalid type argument.  Type must be one of 'expression', 'colourScale', 'databar', 'duplicates', 'beginsWith', 'endsWith', 'contains' or 'notContains'"
@@ -514,6 +562,74 @@ conditionalFormatting <-
       }
 
       invisible(dxfId <- wb$addDXFS(style))
+    } else if (type == "topN") {
+      # type == "topN"
+      # - rule is ignored
+      # - 'rank' and 'percent' are named params 
+      
+      if (is.null(style)) {
+        style <-
+          createStyle(fontColour = "#9C0006", bgFill = "#FFC7CE")
+      } 
+      
+      if (!"Style" %in% class(style)) {
+        stop("If type == 'topN', style must be a Style object.")
+      } 
+      
+      invisible(dxfId <- wb$addDXFS(style)) 
+      
+      ## Additional parameters passed by ...
+      if ("percent" %in% names(params)) {
+        params$percent <- as.integer(params$percent)
+        if (is.na(params$percent)) {
+          stop("percent must be 0/1 or TRUE/FALSE")
+        }
+      } 
+      
+      if ("rank" %in% names(params)) {
+        params$rank <- as.integer(params$rank)
+        if (is.na(params$rank)) {
+          stop("rank must be a number")
+        }
+      } 
+      
+      invisible(dxfId <- wb$addDXFS(style))
+      values <- params
+      rule <- style
+    } else if (type == "bottomN") {
+      # type == "bottomN"
+      # - rule is ignored
+      # - 'rank' and 'percent' are named params 
+      
+      if (is.null(style)) {
+        style <-
+          createStyle(fontColour = "#9C0006", bgFill = "#FFC7CE")
+      } 
+      
+      if (!"Style" %in% class(style)) {
+        stop("If type == 'bottomN', style must be a Style object.")
+      }
+      
+      invisible(dxfId <- wb$addDXFS(style)) 
+      
+      ## Additional parameters passed by ...
+      if ("percent" %in% names(params)) {
+        params$percent <- as.integer(params$percent)
+        if (is.na(params$percent)) {
+          stop("percent must be 0/1 or TRUE/FALSE")
+        }
+      } 
+      
+      if ("rank" %in% names(params)) {
+        params$rank <- as.integer(params$rank)
+        if (is.na(params$rank)) {
+          stop("rank must be a number")
+        }
+      } 
+      
+      invisible(dxfId <- wb$addDXFS(style))
+      values <- params
+      rule <- style
     }
 
 
