@@ -121,6 +121,13 @@ openxlsxCoerce.summary.prcomp <- function(x, rowNames) {
 
 
 
+#' @name openxlsxCoerce.survdiff
+#' @description like print.survdiff with some ideas from the ascii package
+#' @param x data.frame for input
+#' @param rowNames rownames
+#' @importFrom stats pchisq
+#' @keywords internal
+#' @noRd
 openxlsxCoerce.survdiff <- function(x, rowNames) {
 
 
@@ -196,7 +203,7 @@ openxlsxCoerce.coxph <- function(x, rowNames) {
 openxlsxCoerce.summary.coxph <- function(x, rowNames) {
   coef <- x$coefficients
   ci <- x$conf.int
-  nvars <- nrow(coef)
+  # nvars <- nrow(coef)  variable not used
 
   tmp <- cbind(
     coef[, -ncol(coef), drop = FALSE], # p later
