@@ -80,7 +80,7 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
       }
       
       if(sheetPr.size() == 0)
-        sheetPr = getChildlessNode(xml_pre, "<sheetPr");
+        sheetPr = getChildlessNode(xml_pre, "sheetPr");
       
       if(sheetPr.size() > 0)
         this_worksheet.field("sheetPr") = sheetPr;
@@ -88,7 +88,7 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
       
       
       // Freeze Panes
-      CharacterVector node_xml = getChildlessNode(xml_pre, "<pane ");
+      CharacterVector node_xml = getChildlessNode(xml_pre, "pane");
       if(node_xml.size() > 0)
         this_worksheet.field("freezePane") = node_xml;
       
@@ -103,21 +103,23 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
       if(cols.size() > 0){
         
         NumericVector widths;
-        IntegerVector columns;
+        IntegerVector columns_with_widths;
+        IntegerVector columns_with_groups;
         CharacterVector column_hidden;
         CharacterVector col_outline;
-        // CharacterVector col_hidden;
+        CharacterVector col_hidden;
         
-        for(size_t ci = 0; ci < cols.size(); ci++){
+        for (size_t ci = 0; ci < cols.size(); ci++) {
           
           double tmp_width = 0;
           std::string tmp_hidden;
           int min_c = 0;
           int max_c = 0;
           std::string tmp_coloutline;
-          // std::string tmp_col_hidden;
+
           buf = cols[ci];
-          if((buf.find("customWidth", 0) != string::npos) | (buf.find("outlineLevel", 0) != string::npos)){
+          // If either custom widths or groupings, get column index
+          if ((buf.find("customWidth", 0) != string::npos) | (buf.find("outlineLevel", 0) != string::npos)) {
             
             tmp_pos = buf.find("min=\"", 0);
             endPos = buf.find(tagEnd, tmp_pos + 5);
@@ -126,65 +128,79 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
             tmp_pos = buf.find("max=\"", 0);
             endPos = buf.find(tagEnd, tmp_pos + 5);
             max_c = atoi(buf.substr(tmp_pos + 5, endPos - tmp_pos - 5).c_str());
-            
-            tmp_pos = buf.find("width=\"", 0);
-            endPos = buf.find(tagEnd, tmp_pos + 7);
-            tmp_width = atof(buf.substr(tmp_pos + 7, endPos - tmp_pos - 7).c_str()) - 0.71;
-            
+
             tmp_pos = buf.find("hidden=\"", 0);
 
-            if(tmp_pos != string::npos){
+            if (tmp_pos != string::npos) {
               endPos = buf.find(tagEnd, tmp_pos + 8);
               tmp_hidden = buf.substr(tmp_pos + 8, endPos - tmp_pos - 8);
-            }else{
+            } else {
               tmp_hidden = "0";
             }
 
-            tmp_pos = buf.find("outlineLevel=\"", 0);
-            endPos = buf.find(tagEnd, tmp_pos + 14);
-            tmp_coloutline = buf.substr(tmp_pos + 14, endPos - tmp_pos - 14);
-            
+            // If column has both a custom width and is part of a group
+            if ((buf.find("customWidth", 0) != string::npos) & (buf.find("outlineLevel", 0) != string::npos)) {
+              tmp_pos = buf.find("width=\"", 0);
+              endPos = buf.find(tagEnd, tmp_pos + 7);
+              tmp_width = atof(buf.substr(tmp_pos + 7, endPos - tmp_pos - 7).c_str()) - 0.71;
+              
+              tmp_pos = buf.find("outlineLevel=\"", 0);
+              endPos = buf.find(tagEnd, tmp_pos + 14);
+              tmp_coloutline = buf.substr(tmp_pos + 14, endPos - tmp_pos - 14);  
 
-            while(min_c <= max_c){
-              widths.push_back(tmp_width);
-              columns.push_back(min_c);
-              column_hidden.push_back(tmp_hidden);
-              col_outline.push_back(tmp_coloutline);
-              // col_hidden.push_back(tmp_hidden);
-              min_c++;
+              while (min_c <= max_c) {
+                widths.push_back(tmp_width);
+                columns_with_widths.push_back(min_c);
+                columns_with_groups.push_back(min_c);
+                column_hidden.push_back(tmp_hidden);
+                col_hidden.push_back(tmp_hidden);
+                col_outline.push_back(tmp_coloutline);
+                min_c++;
+              }
+              
+            } else if (buf.find("customWidth", 0) != string::npos) {      // Column only has a custom width
+
+              tmp_pos = buf.find("width=\"", 0);
+              endPos = buf.find(tagEnd, tmp_pos + 7);
+              tmp_width = atof(buf.substr(tmp_pos + 7, endPos - tmp_pos - 7).c_str()) - 0.71;
+
+              while (min_c <= max_c) {
+                widths.push_back(tmp_width);
+                columns_with_widths.push_back(min_c);
+                column_hidden.push_back(tmp_hidden);
+                min_c++;
+              }
+            } else {      // Column is only part of a group
+
+              tmp_pos = buf.find("outlineLevel=\"", 0);
+              endPos = buf.find(tagEnd, tmp_pos + 14);
+              tmp_coloutline = buf.substr(tmp_pos + 14, endPos - tmp_pos - 14);  
+
+              while (min_c <= max_c) {
+                columns_with_groups.push_back(min_c);
+                col_hidden.push_back(tmp_hidden);
+                col_outline.push_back(tmp_coloutline);
+                min_c++;
+              }
             }
-
           }
-
-          // if (buf.find("outlineLevel", 0) != string::npos) {
-
-          //   tmp_pos = buf.find("outlineLevel=\"", 0);
-          //   endPos = buf.find(tagEnd, tmp_pos + 14);
-          //   tmp_coloutline = buf.substr(tmp_pos + 14, endPos - tmp_pos - 14);
-          //   col_outline.push_back(tmp_coloutline);
-          //   col_hidden.push_back(tmp_hidden);
-          // }
         }
         
-        if(widths.size() > 0){
+        if (widths.size() > 0) {
           CharacterVector tmp_widths(widths);
-          tmp_widths.attr("names") = columns;
+          tmp_widths.attr("names") = columns_with_widths;
           tmp_widths.attr("hidden") = column_hidden;
           colWidths[i] = tmp_widths;
         }
 
-        if(col_outline.size() > 0) {
+        if (col_outline.size() > 0) {
           CharacterVector columns_outline(col_outline);
-          columns_outline.attr("names") = columns;
-          columns_outline.attr("hidden") = column_hidden;
+          columns_outline.attr("names") = columns_with_groups;
+          columns_outline.attr("hidden") = col_hidden;
           colOutlineLevels[i] = columns_outline;
         }
-        
       }
       
-      
-      
-      
       /* --- Everything after sheetData --- */
       size_t pos_post = 0;
       if(has_data){
@@ -195,28 +211,28 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
       
       std::string xml_post = xml.substr(pos_post);
       
-      node_xml = getChildlessNode(xml_post, "<sheetProtection ");
+      node_xml = getChildlessNode(xml_post, "sheetProtection");
       if(node_xml.size() > 0) {
         this_worksheet.field("sheetProtection") = node_xml;
       }
       
       
-      node_xml = getChildlessNode(xml_post, "<autoFilter ");
+      node_xml = getChildlessNode(xml_post, "autoFilter");
       if(node_xml.size() > 0)
         this_worksheet.field("autoFilter") = node_xml;
       
       
-      node_xml = getChildlessNode(xml_post, "<hyperlink ");
+      node_xml = getChildlessNode(xml_post, "hyperlink");
       if(node_xml.size() > 0)
         this_worksheet.field("hyperlinks") = node_xml;
       
       
-      node_xml = getChildlessNode(xml_post, "<pageMargins ");
+      node_xml = getChildlessNode(xml_post, "pageMargins");
       if(node_xml.size() > 0)
         this_worksheet.field("pageMargins") = node_xml;
       
       
-      node_xml = getChildlessNode(xml_post, "<pageSetup ");
+      node_xml = getChildlessNode(xml_post, "pageSetup");
       if(node_xml.size() > 0){
         for(int j = 0; j < node_xml.size(); j++){
           
@@ -236,7 +252,7 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
         this_worksheet.field("pageSetup") = node_xml;
       }
       
-      node_xml = getChildlessNode(xml_post, "<mergeCell ");
+      node_xml = getChildlessNode(xml_post, "mergeCell");
       if(node_xml.size() > 0)
         this_worksheet.field("mergeCells") = node_xml;
       
@@ -281,9 +297,9 @@ SEXP loadworksheets(Reference wb, List styleObjects, std::vector<std::string> xm
       }
       
       
-      node_xml = getChildlessNode(xml_post, "<drawing ");
+      node_xml = getChildlessNode(xml_post, "drawing");
       if(node_xml.size() == 0)
-        node_xml = getChildlessNode(xml_post, "<legacyDrawing ");
+        node_xml = getChildlessNode(xml_post, "legacyDrawing");
       
       if(node_xml.size() > 0){
         for(int j = 0; j < node_xml.size(); j++){
@@ -762,8 +778,8 @@ SEXP getNodes(std::string xml, std::string tagIn){
     
   }  
   
-  return wrap(r) ;  
-  
+  CharacterVector out = wrap(r);  
+  return markUTF8(out);
 }
 
 
@@ -798,9 +814,8 @@ SEXP getOpenClosedNode(std::string xml, std::string open_tag, std::string close_
     
   }  
   
-  return wrap(r) ;  
-  
-  
+  CharacterVector out = wrap(r);  
+  return markUTF8(out);
 }
 
 
@@ -836,7 +851,7 @@ SEXP getAttr(CharacterVector x, std::string tag){
     }
   }
   
-  return wrap(r) ;  
+  return markUTF8(r);   // no need to wrap as r is already a CharacterVector
   
 }
 
@@ -871,32 +886,57 @@ std::vector<std::string> getChildlessNode_ss(std::string xml, std::string tag){
 
 
 // [[Rcpp::export]]
-CharacterVector getChildlessNode(std::string xml, std::string tag){
+CharacterVector getChildlessNode(std::string xml, std::string tag) {
   
-  size_t k = tag.length();
+  // size_t k = tag.length(); variable not used
   if(xml.length() == 0)
     return wrap(NA_STRING);
   
-  xml = " " + xml;
+  size_t begPos = 0, endPos = 0;
   
   std::vector<std::string> r;
-  size_t pos = 0;
-  size_t endPos = 0;
-  std::string tagEnd = "/>";
+  std::string res = "";
   
-  while(1){
+  // check "<tag "
+  std::string begTag = "<" + tag + " ";
+  std::string endTag = ">";
+  
+  // initial check, which kind of tags to expect
+  begPos = xml.find(begTag, begPos);
+  
+  // if begTag was found
+  if(begPos != std::string::npos) {
     
-    pos = xml.find(tag, pos+1);    
-    if(pos == std::string::npos)
-      break;
+    endPos = xml.find(endTag, begPos);
+    res = xml.substr(begPos, (endPos - begPos) + endTag.length());
     
-    endPos = xml.find(tagEnd, pos+k);
-    
-    r.push_back(xml.substr(pos, endPos-pos+2).c_str());
+    // check if last 2 characters are "/>"
+    // <foo/> or <foo></foo>
+    if (res.substr( res.length() - 2 ).compare("/>") != 0) {
+      // check </tag>
+      endTag = "</" + tag + ">";
+    }
     
+    // try with <foo ... />
+    while( 1 ) {
+      
+      begPos = xml.find(begTag, begPos);
+      endPos = xml.find(endTag, begPos);
+      
+      if(begPos == std::string::npos) 
+        break;
+      
+      // read from initial "<" to final ">"
+      res = xml.substr(begPos, (endPos - begPos) + endTag.length());
+      
+      begPos = endPos + endTag.length();
+      r.push_back(res);
+    }
   }
   
-  return wrap(r) ;  
+  
+  CharacterVector out = wrap(r);  
+  return markUTF8(out);
   
 }
 
@@ -944,7 +984,8 @@ CharacterVector get_extLst_Major(std::string xml){
     
   }
   
-  return wrap(r) ;  
+  CharacterVector out = wrap(r);  
+  return markUTF8(out);
   
 }
 
