@@ -157,8 +157,8 @@ List getCellInfo(std::string xmlFile,
   // file.open(xmlFile.c_str());
   // while (file >> buf)
   // xml += buf + ' ';
-  std::string xml = read_file_newline(xmlFile);
-  
+  std::string xml = read_file_newline(xmlFile);  
+  std::string xml2 = "";
   std::string rtag = "r=";
   std::string ttag = " t=";
   std::string stag = " s=";
@@ -182,6 +182,7 @@ List getCellInfo(std::string xmlFile,
   }
   
   xml = xml.substr(pos + 11);     // get from "sheedData" to the end
+  xml2 = xml;
   
   // startRow cut off
   int row_i = 0;
@@ -275,7 +276,7 @@ List getCellInfo(std::string xmlFile,
   }
   
   // pull out cell merges
-  CharacterVector merge_cell_xml = getChildlessNode(xml, "<mergeCell ");
+  CharacterVector merge_cell_xml = getChildlessNode(xml2, "mergeCell");
   
   
   CharacterVector r(ocs);
@@ -423,6 +424,7 @@ SEXP read_workbook(IntegerVector cols_in,
                    IntegerVector string_inds,
                    LogicalVector is_date,
                    bool hasColNames,
+                   char hasSepNames,
                    bool skipEmptyRows,
                    bool skipEmptyCols,
                    int nRows,
@@ -490,7 +492,7 @@ SEXP read_workbook(IntegerVector cols_in,
   if(hasColNames){
     
     int row_1 = rows[0];
-    char name[20];
+    char name[6];
     
     IntegerVector row1_inds = which_cpp(rows == row_1);
     IntegerVector header_cols = cols[row1_inds];
@@ -498,18 +500,24 @@ SEXP read_workbook(IntegerVector cols_in,
     LogicalVector missing_header = is_na(header_inds);
     
     // looping over each column
-    for(int i=0; i < nCols; i++){
+    for(unsigned short i=0; i < nCols; i++){
       
       if(missing_header[i]){  // a missing header element
         
-        sprintf(&(name[0]), "X%d", i+1);
+        sprintf(&(name[0]), "X%hu", i+1);
+        // sprintf(&(name[0]), "X%u", i+1);
+        // snprintf(&(name[0]), sizeof(&(name[0])), "X%d", i+1);
+        // snprintf(&(name[0]), 10, "X%d", i+1);
         col_names[i] = name;
         
       }else{  // this is a header elements 
         
         col_names[i] = v[pos];
         if(col_names[i] == "NA"){
-          sprintf(&(name[0]), "X%d", i+1);
+          sprintf(&(name[0]), "X%hu", i+1);
+          // sprintf(&(name[0]), "X%du", i+1);
+          // snprintf(&(name[0]), sizeof(&(name[0])), "X%d", i+1);
+          // snprintf(&(name[0]), 10, "X%d", i+1);
           col_names[i] = name;
         }
         
@@ -520,7 +528,7 @@ SEXP read_workbook(IntegerVector cols_in,
     }
     
     // tidy up column names
-    col_names = clean_names(col_names);
+    col_names = clean_names(col_names,hasSepNames);
     
     //--------------------------------------------------------------------------------
     // Remove elements from rows, cols, v that have been used as headers
@@ -564,9 +572,11 @@ SEXP read_workbook(IntegerVector cols_in,
     
     
   }else{ // else col_names is FALSE
-    char name[20];
+    char name[6];
     for(int i =0; i < nCols; i++){
-      sprintf(&(name[0]), "X%d", i+1);
+      sprintf(&(name[0]), "X%hu", i+1);
+       // snprintf(&(name[0]), sizeof(&(name[0])), "X%d", i+1);
+      // sprintf(&(name[0]), "X%u", i+1);
       col_names[i] = name;
     }
   }
@@ -608,7 +618,7 @@ SEXP read_workbook(IntegerVector cols_in,
   // Rcout << "is_date.size(): " << is_date.size() << endl;
   // Rcout << "v.size(): " << v.size() << endl;
   // Rcout << "has_date: " << has_date << endl;
-
+  
   if(allNumeric){
     
     m = buildMatrixNumeric(v, rows, cols, col_names, nRows, nCols);
@@ -701,4 +711,3 @@ int calc_number_rows(CharacterVector x, bool skipEmptyRows){
   return(nRows);
   
 }
-
